#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>

#include "rc4.h"
#include "sha1.h"

/* packet encryption layer */

struct rc4_state recv_state;
struct rc4_state send_state;

int pel_client_init( int server, char *key )
{
    struct timeval tv;
    struct sha1_context sha1_ctx;
    uint8 IV[40], skey[20], *buffer;

    /* create both recv/send 160-bit IVs (initialization vectors) */

    if( gettimeofday( &tv, NULL ) < 0 )
    {
        return( 1 );
    }

    sha1_starts( &sha1_ctx );
    sha1_update( &sha1_ctx, (uint8 *) &tv, sizeof( tv ) );
    sha1_finish( &sha1_ctx, &IV[0] );

    if( gettimeofday( &tv, NULL ) < 0 )
    {
        return( 1 );
    }

    sha1_starts( &sha1_ctx );
    sha1_update( &sha1_ctx, (uint8 *) &tv, sizeof( tv ) );
    sha1_finish( &sha1_ctx, &IV[20] );

    /* and pass them to the server */

    if( send( server, IV, 40, 0 ) != 40 )
    {
        return( 1 );
    }

    /* now setup the 160-bit session keys, using the SHA-1
     * hash of the IV appended to the secret password */

    sha1_starts( &sha1_ctx );
    sha1_update( &sha1_ctx, (uint8 *) key, strlen( key ) );
    sha1_update( &sha1_ctx, &IV[0], 20 );
    sha1_finish( &sha1_ctx, skey );

    rc4_setup( &recv_state, skey, 20 );

    sha1_starts( &sha1_ctx );
    sha1_update( &sha1_ctx, (uint8 *) key, strlen( key ) );
    sha1_update( &sha1_ctx, &IV[20], 20 );
    sha1_finish( &sha1_ctx, skey );

    rc4_setup( &send_state, skey, 20 );

    /* drop the first 1024 bytes of RC4's output */

    if( ! ( buffer = (uint8 *) malloc( 1024 ) ) )
    {
        return( 1 );
    }

    rc4_crypt( &recv_state, buffer, 1024 );
    rc4_crypt( &send_state, buffer, 1024 );

    free( buffer );

    return( 0 );
} 

int pel_server_init( int client, char *key )
{
    struct sha1_context sha1_ctx;
    uint8 IV[40], skey[20], *buffer;

    /* get both send/recv 160-bit IVs from the client */

    if( recv( client, IV, 40, 0 ) != 40 )
    {
        return( 1 );
    }

    /* setup the 160-bit session keys */

    sha1_starts( &sha1_ctx );
    sha1_update( &sha1_ctx, (uint8 *) key, strlen( key ) );
    sha1_update( &sha1_ctx, &IV[20], 20 );
    sha1_finish( &sha1_ctx, skey );

    rc4_setup( &recv_state, skey, 20 );

    sha1_starts( &sha1_ctx );
    sha1_update( &sha1_ctx, (uint8 *) key, strlen( key ) );
    sha1_update( &sha1_ctx, &IV[0], 20 );
    sha1_finish( &sha1_ctx, skey );

    rc4_setup( &send_state, skey, 20 );

    /* drop the first 1024 bytes of RC4's output */

    if( ! ( buffer = (uint8 *) malloc( 1024 ) ) )
    {
        return( 1 );
    }

    rc4_crypt( &recv_state, buffer, 1024 );
    rc4_crypt( &send_state, buffer, 1024 );

    free( buffer );

    return( 0 );
}

/* encrypted recv wrapped */

int pel_recv( int s, void *buf, size_t len, int flags )
{
    int n = recv( s, buf, len, flags );
    rc4_crypt( &recv_state, buf, n );
    return( n );
}

/* encrypted send wrapped */

int pel_send( int s, void *msg, size_t len, int flags )
{
    rc4_crypt( &send_state, msg, len );
    return( send( s, msg, len, flags ) );
}
